﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml;
using System.Xml.Linq;
using System.Windows.Browser;

namespace CoffeeMap
{
    [ScriptableType()]
    public partial class MainPage : UserControl
    {

        IEnumerable<Area> areas;
        IEnumerable<Shop> shops;

        public MainPage()
        {
            InitializeComponent();
            LoadArea();
            LoadShop();
            HtmlPage.RegisterScriptableObject("Page", this);
            treeView1.ItemsSource = areas;
        }

        public void LoadArea()
        {
            XDocument AreaXml = XDocument.Load("Area.xml");
            areas = from area in AreaXml.Element("arealist").Elements("area")
                    select new Area(area.Attribute("code").Value, area.Attribute("name").Value,
                                                      (from area2 in area.Elements("area")
                               select new Area(area2.Attribute("code").Value, area2.Attribute("name").Value, null)
                               ));
        }

        public void LoadShop()
        {
            XDocument shopXml = XDocument.Load("Shoplist.xml");
            shops = from shop in shopXml.Descendants("shop")
                    where (shop.Parent.Name == "area2" || shop.Parent.Name == "area1") && shop.Descendants("location").ToArray().Count() != 0
                    select new Shop(shop.Element("id").Value,
                        shop.Element("name").Value,
                        shop.Element("address").Value,
                        shop.Element("tel").Value, shop.Element("time").Value,
                        shop.Parent.Attribute("code").Value,
                        shop.Element("location").Value);
        }

        private void treeView1_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            Area a = treeView1.SelectedItem as Area;
            listBox1.ItemsSource = from Shop s in shops
                                   where s.Area == a.Code
                                   select s;
        }

        private void listBox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Shop sh = listBox1.SelectedItem as Shop;

            if (sh != null)
            {
                HtmlPage.RegisterScriptableObject("mark", sh);
                HtmlPage.Window.Invoke("mapMove", sh);
                ShopDisplay(sh.Id);
            }
        }

        [ScriptableMember()]
        public void BoundChange(double lat1, double lat2, double lng1, double lng2)
        {
            double lats = Math.Min(lat1, lat2);
            double latl = Math.Max(lat1, lat2);
            double lngs = Math.Min(lng1, lng2);
            double lngl = Math.Max(lng1, lng2);

            double latc = (lats + latl) / 2;
            double lngc = (lngs + lngl) / 2;

            var list1 = (from s in shops
                      where s.CenterLat <= latl && s.CenterLat >= lats &&
                          s.CenterLng <= lngl && s.CenterLng >= lngs
                      orderby Math.Pow(s.CenterLat - latc, 2) +
                          Math.Pow(s.CenterLng - lngc, 2)
                      select s).Take(100).ToList();

            shopList list2 = new shopList();

            for (int i = 0; i < list1.Count; i++)
                list2.Add(list1[i]);

            HtmlPage.RegisterScriptableObject("maps", list2);
            HtmlPage.Window.Invoke("displayShopList", list2);
        }

        private double Distance(double lat1, double lat2, double lng1, double lng2)
        {
            double d = Math.Pow(lat1 - lat2, 2) + Math.Pow(lng1 - lng2, 2);
            return d;
        }

        [ScriptableMember()]
        public void ShopDisplay(string id)
        {
            var myShop = (from s in shops
                      where s.Id == id
                      select s).ToList();
            shopGrid.DataContext = myShop[0];
        }
    }

    public class Area
    {
        public string Code { get; set; }
        public string Name { get; set; }
        public List<Area> Children { get; set; }

        public Area(string c, string n, IEnumerable<Area> l)
        {
            Code = c;
            Name = n;
            if (l == null)
                Children = null;
            else
                Children = l.ToList();
        }
    }

    [ScriptableType]
    public class shopList : List<Shop>, IList<Shop>, IEnumerable<Shop>
    {
        public shopList() { }
    }

    [ScriptableType]
    public class Shop
    {
        [ScriptableMember]
        public string Id { get; set; }

        [ScriptableMember]
        public string Name { get; set; }

        [ScriptableMember]
        public double CenterLat { get; set; }

        [ScriptableMember]
        public double CenterLng { get; set; }

        public string Address { get; set; }
        public string Tel { get; set; }
        public string Time { get; set; }
        public string Area { get; set; }

        public Shop(string i, string n, string a, string t, string tm, string cd,string ce)
        {
            Id = i;
            Name = n;
            Address = a;
            Tel = t;
            Time = splitTrim(tm);
            Area = cd;
            string[] d = ce.Split(',');
            CenterLat = double.Parse(d[0]);
            CenterLng = double.Parse(d[1]);
        }

        private string splitTrim(string s)
        {
            string[] t = s.Split(new string[] {"\r\n", "\r", "\n"}, StringSplitOptions.RemoveEmptyEntries);
            string mm = "";
            for (int i = 0; i < t.GetLength(0); i++)
            {
                mm += t[i].Trim() + "\r\n";
            }
            return mm;
        }
    }
}
